
# q-expansion of newform 1680.2.bg.t, downloaded from the LMFDB on 27 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1680
weight = 2
poly_data = [4, 0, 2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 0, 1, 0], 2], [[0, 4, 0, 1], 2], [[0, 2, 0, -1], 2]]

hecke_ring_character_values = [[1471, [1, 0, 0, 0]], [421, [1, 0, 0, 0]], [1121, [1, 0, 0, 0]], [337, [1, 0, 0, 0]], [241, [0, 1, 0, 0]]]
aps_data = [[0, 0, 0, 0], [1, 1, 0, 0], [0, -1, 0, 0], [0, -1, 0, 1], [0, 0, 1, 1], [-1, 0, 1, -2], [0, 0, 1, 1], [0, 7, 0, 0], [0, 0, -2, 1], [-6, 0, 1, -2], [-1, -1, 2, 2], [0, -1, -2, 1], [0, 0, -1, 2], [1, 0, -1, 2], [0, 6, 0, 0], [0, 0, 2, 2], [-6, -6, 1, 1], [0, -4, -4, 2], [-1, -1, 1, 1], [0, 0, 3, -6], [-5, -5, 1, 1], [0, -11, 0, 0], [6, 0, 1, -2], [0, -6, 2, -1], [8, 0, -2, 4], [12, 12, 1, 1], [0, 13, -2, 1], [0, 0, 6, -3], [1, 1, -2, -2], [-18, 0, 0, 0], [7, 0, -1, 2], [0, -6, 4, -2], [0, 0, 1, 1], [7, 0, 2, -4], [0, -12, 0, 0], [14, 14, -2, -2], [-2, -2, -2, -2], [0, -8, 0, 0], [-6, 0, -3, 6], [0, -12, 4, -2], [-6, -6, 0, 0], [-13, 0, 0, 0], [0, -6, 0, 0], [-11, -11, -1, -1], [-12, 0, -1, 2], [2, 2, -2, -2], [-14, 0, -2, 4], [16, 0, -2, 4], [6, 6, 5, 5], [0, -7, 0, 0], [0, 6, -4, 2], [6, 0, -4, 8], [4, 4, 0, 0], [6, 0, 3, -6], [0, 6, -2, 1], [0, 0, 2, 2], [-6, -6, -2, -2], [0, -2, -12, 6], [-5, -5, 3, 3], [12, 0, 4, -8], [11, 11, -3, -3], [12, 0, -4, 8], [1, 0, -1, 2], [0, 0, -5, -5], [0, 11, -6, 3], [0, -12, 6, -3], [0, -17, 0, 0], [-1, 0, -3, 6], [-24, -24, 0, 0], [-10, 0, 0, 0], [6, 6, -1, -1], [0, -6, -2, 1], [23, 23, -3, -3], [0, 5, -6, 3], [-5, 0, 6, -12], [0, 0, 12, -6], [0, 0, -5, -5], [0, -13, -2, 1], [0, -12, -4, 2], [-17, -17, 0, 0], [6, 0, 2, -4], [23, 0, 2, -4], [-6, -6, -6, -6], [29, 0, 1, -2], [0, 10, 0, 0], [0, -12, 4, -2], [-6, 0, 0, 0], [0, 11, -10, 5], [6, 0, 3, -6], [-23, 0, 3, -6], [0, -6, -8, 4], [12, 12, 0, 0], [-25, -25, 3, 3], [-24, 0, 2, -4], [0, -11, 4, -2], [18, 0, 2, -4], [0, 6, 12, -6], [-12, -12, -6, -6], [0, 1, -2, 1], [0, -7, 16, -8], [-8, 0, -6, 12], [30, 30, 0, 0], [-6, -6, 0, 0], [0, -12, 14, -7], [5, 5, 8, 8], [-17, -17, -1, -1], [-24, 0, -5, 10], [0, -18, -10, 5], [-24, -24, 2, 2], [5, 0, -2, 4], [0, 25, -2, 1], [-20, -20, -6, -6], [-18, 0, -2, 4], [5, 5, 4, 4], [-8, 0, 0, 0], [-30, -30, 1, 1], [7, 0, 3, -6], [6, 6, -1, -1], [0, 6, -2, 1], [-24, 0, -4, 8], [19, 19, 4, 4], [17, 0, 3, -6], [0, 0, -6, 3], [12, 12, -5, -5], [0, 7, 16, -8], [30, 0, -5, 10], [0, -28, 4, -2], [0, -18, 4, -2], [-5, 0, 1, -2], [0, 23, 10, -5], [-7, -7, 2, 2], [6, 0, -5, 10], [0, -11, 16, -8], [26, 0, -4, 8], [0, 42, -2, 1], [5, 0, 0, 0], [18, 18, 7, 7], [20, 20, -2, -2], [-36, 0, 1, -2], [6, 6, 8, 8], [22, 0, -8, 16], [0, -24, -14, 7], [-22, -22, 4, 4], [-18, 0, -6, 12], [-11, -11, 0, 0], [-30, 0, 1, -2], [-13, 0, -3, 6], [0, 0, 1, 1], [0, 22, 0, 0], [0, 30, 4, -2], [0, -10, 0, 0], [24, 0, 4, -8], [7, 0, -3, 6], [0, 6, 10, -5], [-43, -43, -3, -3], [-30, 0, -5, 10], [0, -17, 8, -4], [0, 24, -6, 3], [11, 0, -7, 14], [18, 18, -7, -7], [0, -18, -10, 5], [-36, 0, 2, -4], [31, 0, 1, -2], [0, -36, 8, -4], [30, 30, 5, 5], [-12, -12, 7, 7], [-19, -19, -8, -8], [13, 13, -3, -3], [-19, 0, 2, -4], [0, -24, 18, -9], [24, 24, 2, 2], [-4, 0, 0, 0], [0, 30, 2, -1], [1, 1, 13, 13], [-1, -1, -4, -4], [-12, 0, -12, 24], [7, 0, 2, -4], [-6, -6, -10, -10], [-20, 0, 4, -8], [0, -1, 16, -8], [0, -47, -6, 3], [-6, 0, 9, -18], [11, 0, 1, -2], [0, -24, 12, -6], [6, 6, 8, 8], [36, 36, -2, -2], [-14, -14, -2, -2], [29, 29, -3, -3], [0, -1, -4, 2], [12, 12, -1, -1], [0, 11, -6, 3], [30, 0, -1, 2], [0, 40, 0, 0], [0, -12, 0, 0], [0, 0, 9, 9], [-12, -12, 1, 1], [-23, -23, 0, 0], [0, -28, 12, -6], [18, 0, 6, -12], [0, 48, -6, 3], [30, 30, 2, 2], [-29, 0, 6, -12], [0, -22, 12, -6], [28, 28, 6, 6], [0, 0, -11, 22], [-30, -30, -6, -6], [0, 16, -16, 8], [0, 18, -16, 8], [-60, 0, -2, 4], [-34, -34, 0, 0], [0, -31, -6, 3], [-6, 0, -5, 10], [16, 0, 2, -4], [0, 12, 8, -4], [-18, -18, 5, 5], [0, -19, 20, -10], [-16, -16, -4, -4], [-24, -24, 4, 4], [0, 0, 12, -6], [-42, 0, 4, -8], [0, -1, -16, 8], [-29, 0, 4, -8], [0, -48, -6, 3], [0, 16, 24, -12], [6, 0, -10, 20], [8, 0, -10, 20], [0, -60, 0, 0], [6, 6, -6, -6], [0, 43, 6, -3], [0, -36, -2, 1], [-5, -5, -5, -5], [20, 20, 12, 12], [49, 0, 2, -4], [60, 60, -3, -3], [13, 0, 11, -22], [-6, -6, -1, -1], [0, 5, -12, 6], [0, 54, 6, -3], [12, 0, 0, 0], [0, 0, -3, 6], [18, 18, -3, -3], [0, 19, -24, 12], [-31, -31, 7, 7], [0, -40, -8, 4], [0, 0, -6, 12], [0, 12, -4, 2], [-5, 0, 7, -14], [36, 36, 1, 1], [-10, -10, -8, -8], [36, 0, -8, 16], [-25, 0, 9, -18], [0, -48, -8, 4], [18, 18, -12, -12], [-25, 0, 2, -4], [48, 48, -2, -2], [0, -54, -2, 1], [-35, -35, 8, 8], [-1, -1, -7, -7], [-42, 0, 3, -6], [0, 2, -24, 12], [-19, -19, 9, 9], [-24, 0, 10, -20], [-5, -5, -4, -4], [29, 0, -7, 14], [-18, -18, 6, 6], [0, 1, -16, 8], [36, 0, 1, -2], [18, 0, 8, -16], [22, 0, 0, 0], [-42, -42, 3, 3], [0, -31, -20, 10], [-34, -34, 0, 0], [19, 19, 1, 1], [0, 19, -20, 10], [-10, 0, 2, -4], [0, -8, 28, -14], [0, 18, 18, -9], [-11, -11, -8, -8], [0, 17, 24, -12], [0, 0, 0, 0], [24, 24, 10, 10], [-13, -13, 2, 2], [-6, 0, 0, 0], [-7, 0, 14, -28], [0, -8, -28, 14], [0, 12, -30, 15], [31, 31, -5, -5], [-6, 0, 9, -18], [-55, -55, -2, -2], [-24, 0, 11, -22], [48, 48, 2, 2], [-12, -12, -10, -10], [0, -6, -34, 17], [-6, 0, -9, 18], [23, 0, -1, 2], [-18, -18, 12, 12], [0, 7, 4, -2], [-24, 0, -13, 26], [0, 36, -10, 5], [-29, 0, -3, 6], [0, 14, 0, 0], [0, -42, -4, 2], [26, 26, -4, -4]]
